# Generated by sila2.code_generator; sila2.__version__: 0.12.2
from __future__ import annotations

from typing import List, NamedTuple


class OpenGripper_Responses(NamedTuple):

    pass


class CloseGripper_Responses(NamedTuple):

    pass


class InitializeArm_Responses(NamedTuple):

    Success: bool
    """
    Indicates whether the initialization was successful.
    """


class CloseArm_Responses(NamedTuple):

    Success: bool
    """
    Indicates whether the shutdown was successful.
    """


class ResetArm_Responses(NamedTuple):

    Success: bool
    """
    Indicates whether the reset was successful.
    """


class SetManualMode_Responses(NamedTuple):

    Success: bool
    """
    Indicates whether the arm was successfully switched to manual mode.
    """


class MoveJoints_Responses(NamedTuple):

    MoveJointsResponses: List[float]
    """
    Response indicating the success of the move and the current joint angles.
    """


class StorePosition_Responses(NamedTuple):

    PositionIdentifier: int
    """
    The position in the list of stored poses
    """


class MoveToStoredPosition_Responses(NamedTuple):

    pass
